// ************************************************
//	SmVR_CGeometricObject
//
//
//  Author: Marcio Serolli Pinho
//	Email: pinho@inf.pucrs.br
//	Homepages:
//		http://www.smallvr.org
//		http://grv.inf.pucrs.br
//		http://www.inf.pucrs.br/~pinho
//
//	Porto Alegre, Brazil, 2011.
// ************************************************
#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>

#include "SmVR_CGeometricObject.h"

//#include "SmVR_Scene.h"


static SmVR_CGeometricObject *SMVR_ROOT_GRAPH = NULL;
//static SmVR_Scene *SMVR_SCENE = NULL;

// ************************************************
//	Method: Init
//	Description: private methods that initialize all
//	common object atributes
//
//
// ************************************************
void SmVR_CGeometricObject::Init(char const  *name)
{
	if (strcmp(name, "SMVR_ROOT_GRAPH"))
	{
		TestNullSceneGraph(this);
	}

	child = NULL;
	brother = NULL;
	prevBrother = NULL;
	parent = SMVR_ROOT_GRAPH; // all object has a parent

	RenderFunction = NULL;
	FunctionReturnValue = 0;
	RenderFunctionData = NULL;


	Filename = NULL;
	GeometryData = NULL;

	visible = TRUE;

	if (strlen(name))
	{
		this->name = strdup(name);
	}

	SmVR_SetIdentityMatrix(TransformationMatrix);
	SmVR_SetIdentityMatrix(GlobalTransformationMatrix);

}
// ************************************************
//	Method: TestNullSceneGraph(this)
//	Description:
//
// ************************************************
void SmVR_CGeometricObject::TestNullSceneGraph(SmVR_CGeometricObject *obj)
{
	if (!SMVR_ROOT_GRAPH)
	{
		printf(">>>>>>NULL SCENE GRAPH!!\n");
		exit(1);
	}
	if (!obj)
	{
		printf("NULL object being manipulated.\n");
		printf("Press ENTER to finish.\n");
		getchar();
		exit(1);
	}
}

// ************************************************
//	Method: Object CONSTRUCTOR
//	Description: to create an object without any
//	geometry
//
// ************************************************
SmVR_CGeometricObject::SmVR_CGeometricObject(char const *name)
{
	Init(name);
	RenderFunction = NULL;
	ObjFromFile = NULL;

}

// ************************************************
//	Method: GetName()
//	Description:
//
//
// ************************************************
char *SmVR_CGeometricObject::GetName()
{
	return this->name;
}
// ************************************************
//	Method: GetName()
//	Description:
//
//
// ************************************************
char *SmVR_CGeometricObject::GetFilename()
{
    return Filename;
};


// ************************************************
//	Method: SetName()
//	Description:
//
//
// ************************************************
void SmVR_CGeometricObject::SetName(char const *name)
{
    if (strlen(name))
	{
		this->name = strdup(name);
	}
}
// ************************************************
//	Method: Object CONSTRUCTOR
//	Description: to create an object with a function
//	to describe the geometry
//
// ************************************************
SmVR_CGeometricObject::SmVR_CGeometricObject(char const *name, FunctionPointer *f)
{

	Init(name);

	//printf("Storing function pointer for rendering....\n");
    RenderFunction = f;
	ObjFromFile = NULL;


}
// ************************************************
//	Method:
//	Description:
//
// ************************************************
SmVR_CGeometricObject::SmVR_CGeometricObject(char const *name, SmVR_CObjFromFile *obj)
{
	Init(name);

	if (obj)
	{
		//printf("Receiving a File object to render\n");
		ObjFromFile = obj;
		RenderFunction = NULL;
	}

}

// ************************************************
//	Method:
//	Description:
//
// ************************************************
int SmVR_CGeometricObject::IsVisible(void)
{
	return visible;
}

// ************************************************
//	Method: GetRenderFunctionReturnValue(void)
//	Description: allows to get the last return
//	value from the function that render the object
//
// ************************************************
int SmVR_CGeometricObject::GetRenderFunctionReturnValue(void)
{
	TestNullSceneGraph(this);
	return FunctionReturnValue;
}
// ************************************************
//	Method: SetRenderFunctionData(void *d)
//	Description: allows to specify a pointer
//	that can be used inside the rendering function
//
// ************************************************
void SmVR_CGeometricObject::SetRenderFunctionData(void *d)
{
	TestNullSceneGraph(this);
	RenderFunctionData = d;
}
void SmVR_CGeometricObject::CopyRenderFunctionFromObject(SmVR_CGeometricObject *o)
{
	RenderFunction = o->RenderFunction;
	ObjFromFile = o->ObjFromFile;
}
void SmVR_CGeometricObject::SetRenderFunction(FunctionPointer *f)
{
	RenderFunction = f;
}

FunctionPointer *SmVR_CGeometricObject::GetRenderFunction(void)
{
	return RenderFunction;
}
// ************************************************
//	Method: Print
//	Description: Print the object tree.
//
// ************************************************
void SmVR_CGeometricObject::Print()
{

   	TestNullSceneGraph(this);

	printf("Name: %s\n", this->name);
	if (parent)
		printf("\tParent: %s\n", this->parent->name);

	if (child) {
		printf("CHILD:\n");
		child->Print();
	}

	if (brother)
		brother->Print();
}

// ************************************************
//	Method: Render
//	Description: Render the scene from the object
//
// ************************************************
void SmVR_CGeometricObject::Render()
{

   	TestNullSceneGraph(this);

	glPushMatrix();
		// Apply the local transformation matrix
		glMultMatrixf(&TransformationMatrix[0][0]);
		// Render the object
		// Call the rendering function provided by the programmer
		if (visible)
		{

			if (RenderFunction)
 				FunctionReturnValue = (*RenderFunction) (RenderFunctionData);
			else if (ObjFromFile)
					ObjFromFile->Render();
		}
		// Render the child under the same transformation
		if (child)
			child->Render();
	glPopMatrix();

	// Render the brother. Note that the "previousBrother" is not rendered
	if (brother)
		brother->Render();
}

// ************************************************
//	Method: AddChild
//	Description: This method insert an object 'o'
//	as a child of "this".
//	The algorithm is the following:
//		- first the GTM of "o" is calculated;
//		- "o" is removed from its the tree;
//		- "o" is inserted as child of "this";
//		- the new TM of "o" is calculated.
//
// ************************************************
BOOL SmVR_CGeometricObject::AddChild(SmVR_CGeometricObject *o)
{

	SmVR_Matrix Inverse, Result;

	TestNullSceneGraph(this);

	if (!o)
	{
		printf("AddChild: NULL object to add.\n");
		return FALSE;
	}

	if (o == SMVR_ROOT_GRAPH)
	{
		printf("AddChild: Trying to add the Root object as a child.\n");
		return FALSE;
	}
	if (o == this)
	{
		printf("AddChild: Parent and child are the same.\n");
		return FALSE;
	}
	// "this" is child of "o" ?
	//if (o->FindObjectInChildren(this))
	if (o->FindChildObject(this->name))
	{
		printf("AddChild: Circular reference ! %s is a child of %s.\n", this->name, o->name);
		return FALSE;
	}

	// Calculate the object GTMs
	o->CalculateGlobalTransformationMatrix();
	// REMOVE "o" from the list
	if (o->brother)
		o->brother->prevBrother = o->prevBrother;
	if (o->prevBrother)
		o->prevBrother->brother = o->brother;
	if (o->parent) // if "o" object already belongs the scene graph
	{
		if (o->parent->child == o) // If I was the 1st one in the parent list
			o->parent->child = o->brother;  // makes my brother the new 1st one
	}
	else {
		printf ("AddChild: The object to be added has a NULL parent!\n");
		return FALSE;
	}

	// Put "o" in its new position in the scene graph
	o->parent = this;
	o->brother = this->child;
	o->prevBrother = NULL;
	if (this->child)
		child->prevBrother = o;
	this->child = o;

	// Calculate the inverse of parent TM
	//SmVR_InvertMatrix(&TransformationMatrix[0][0], &Inverse[0][0]);

    CalculateGlobalTransformationMatrix();
	SmVR_InvertMatrix3 (GlobalTransformationMatrix, Inverse);
	// Calculate the new object TM
	SmVR_MultMatrix (o->GlobalTransformationMatrix, Inverse, Result); // Res = MT * Inv
	SmVR_CopyMatrix (o->TransformationMatrix, Result);

	return TRUE;
}

// ************************************************
//	Method: Remove
//	Description: remove and object from the object
//	tree
// ************************************************
BOOL SmVR_CGeometricObject::Remove(SmVR_CGeometricObject *NewParentForChildren)
{

	SmVR_CGeometricObject *aux;

	TestNullSceneGraph(this);

	if (this == NewParentForChildren)
	{
		printf("Remove: \"Myself\" and \"NewParentForChildren\" are the same.\n");
		return FALSE;
	}

	// reorganize the children of 'this'
	if (child)
	{
		if (NewParentForChildren) // set the then parent for all children
		{
			aux = child->brother;
			while (aux)
			{
				NewParentForChildren->AddChild(aux);
				aux = child->brother;
			}
			NewParentForChildren->AddChild(child);
		}
		else {  // There is NO new parent ! So remove all children !
				aux = child->brother;
				while (aux) {
					aux->Remove(NULL); // Remove all brothers
					aux = child->brother;
				}
				child->Remove(NULL); // Remove the child
		}
	}

	// REMOVE "this" from the list
	if (brother)
		brother->prevBrother = prevBrother;
	if (prevBrother)
		prevBrother->brother = brother;
	if (parent) // if "this" object already belongs the scene graph
	{
		if (parent->child == this) // If I was the 1st one in the parent list
			parent->child = brother;  // makes my brother the new 1st one
	}
	else {
		//printf ("Remove: The object to be removed has a NULL parent!\n");
		//return FALSE;
	}

	if(strlen(name))
		free(name);
	//printf("Remove: Object \"%s\" removed\n",this->name);
	return TRUE;
}

// ************************************************
//	Method: TranslateTo(float tx, float ty, float tz)
//	Description: place the object in a new position
//	in PARENT CS
// ************************************************
void SmVR_CGeometricObject::TranslateToOnOBJCS(float x, float y, float z, SmVR_CGeometricObject *OCS)
{
	TestNullSceneGraph(this);

	SmVR_CPoint Point(x,y,z), Displacement;

	// Get the point (x, y, z) in "this" Coordinate System
	// The result will be the translation to be applied to "this"
	OCS->GetPointInOCS(Point, &Displacement, this);

	// Translate "this"
	TranslateBy(Displacement);

}

// ************************************************
//	Method: TranslateTo(float tx, float ty, float tz)
//	Description: place the object in a new position
//	in PARENT CS
// ************************************************
void SmVR_CGeometricObject::TranslateToOnOBJCS(SmVR_CPoint p, SmVR_CGeometricObject *OCS)
{
	TranslateToOnOBJCS(p.X, p.Y, p.Z, OCS);
}

// ************************************************
//	Method: TranslateBy(float tx, float ty, float tz)
//	Description:  move the object using tx,ty,tz
//	as a vector in LOCAL CS
//
// ************************************************
void SmVR_CGeometricObject::TranslateBy(float tx, float ty, float tz)
{
	TestNullSceneGraph(this);

	//TranslateByOnOBJCS(tx, ty, tz,this);

	TransformationMatrix[3][0] += tx;
	TransformationMatrix[3][1] += ty;
	TransformationMatrix[3][2] += tz;
}
// ************************************************
//	Method: TranslateBy(SmVR_CPoint p)
//	Description: see above function
//
// ************************************************
void SmVR_CGeometricObject::TranslateBy(SmVR_CPoint p)
{
	TranslateBy(p.X, p.Y, p.Z);
}

// ************************************************
//	Method: void TranslateByOnOBJCS(float tx, float ty, float tz, SmVR_CGeometricObject *)
//	Description: perform a translation using OCS coordinate
//	system as reference for the rotation
//
// ************************************************
void SmVR_CGeometricObject::TranslateByOnOBJCS(float tx, float ty, float tz, SmVR_CGeometricObject *OCS)
{
	SmVR_CGeometricObject *temp, *parent;

	TestNullSceneGraph(this);

	if (OCS == this)
	   TranslateBy(tx, ty, tz);

	// create a temporary object
	temp = new SmVR_CGeometricObject("temp");

	// insert it on the object tree
	//SMVR_ROOT_GRAPH->AddChild(temp);

	// copy the GTM of 'OCS' to LTM of 'temp'
	// It makes temp to be exactly in the same position/orientation of OCS
	OCS->CalculateGlobalTransformationMatrix();
	SmVR_CopyMatrix(temp->TransformationMatrix, OCS->GlobalTransformationMatrix);

	// save 'this' parent
	parent = this->parent;

	// Add 'this' object to 'temp' object
	temp->AddChild (this);

	// translate 'this' on its parent CS
	TransformationMatrix[3][0] += tx;
	TransformationMatrix[3][1] += ty;
	TransformationMatrix[3][2] += tz;

	// recover the previous parent of 'this'
	parent->AddChild(this);

	// just for debugging
	CalculateGlobalTransformationMatrix();

	// remove 'temp'
	temp->Remove(NULL);
	delete temp;
}

// ************************************************
//	Method: void TranslateByOnOBJCS(float tx, float ty, float tz, SmVR_CGeometricObject *)
//	Description: see above function
//
// ************************************************
void SmVR_CGeometricObject::TranslateByOnOBJCS(SmVR_CPoint p, SmVR_CGeometricObject *OCS)
{
   TranslateByOnOBJCS(p.X, p.Y, p.Z, OCS);
}
// ************************************************
//	Method: ScaleBy(float ex, float ey, float ez)
//	Description: Apply a scale on the object, in
//	LOCAL CS
//
// ************************************************
void SmVR_CGeometricObject::ScaleBy(float ex, float ey, float ez)
{
	TestNullSceneGraph(this);
	glPushMatrix();
		// set the OpenGL matrix with the current object matrix
		glLoadMatrixf(&TransformationMatrix[0][0]);
		// apply the scaling in the OpenGL matrix
		glScaled(ex, ey, ez);
		// set the object matrix with the new OpenGL matrix
		glGetFloatv(GL_MODELVIEW_MATRIX, &TransformationMatrix[0][0]);
	glPopMatrix();

}
// ************************************************
//	Method: ScaleBy(SmVR_CPoint p)
//	Description: see above function
//
// ************************************************
void SmVR_CGeometricObject::ScaleBy(SmVR_CPoint p)
{
	ScaleBy(p.X, p.Y, p.Z);
}
// ************************************************
//	Method: void ScaleByOnOBJCS(float tx, float ty, float tz, SmVR_CGeometricObject *)
//	Description: perform a scale using OCS coordinate
//	system as reference for scaling
//
// ************************************************
void SmVR_CGeometricObject::ScaleByOnOBJCS(float ex, float ey, float ez, SmVR_CGeometricObject *OCS)
{
	SmVR_CGeometricObject *temp, *parent;

	TestNullSceneGraph(this);

	// create a temporary object
	temp = new SmVR_CGeometricObject("temp");

	// copy the GTM of 'OCS' to LTM of 'temp'
	// It makes temp to be exactly in the same position/orientation of OCS
	OCS->CalculateGlobalTransformationMatrix();
	SmVR_CopyMatrix(temp->TransformationMatrix, OCS->GlobalTransformationMatrix);

	// save 'this' parent
	parent = this->parent;

	// Add 'this' object to 'temp' object
	temp->AddChild (this);

	// scale 'temp', so it scales 'this'
	temp->ScaleBy(ex,ey,ez);

	// recover the previous parent of 'this'
	parent->AddChild(this);

	// remove 'temp'
	temp->Remove(NULL);
}

// ************************************************
//	Method: void ScaleByOnOBJCS(SmVrPoint p, SmVR_CGeometricObject *)
//	Description: see above function
//
// ************************************************
void SmVR_CGeometricObject::ScaleByOnOBJCS(SmVR_CPoint p, SmVR_CGeometricObject *OCS)
{
	ScaleByOnOBJCS(p.X, p.Y, p.Z, OCS);
}
// ************************************************
//	Method: RotateBy(float angle, float ax, float ay, float az)
//	Description: rotate the object by "angle"
//	degrees around the axis specified by (ax, ay, az)
//	vector.
//
// ************************************************
void SmVR_CGeometricObject::RotateBy(float angle, float ax, float ay, float az)
{
	TestNullSceneGraph(this);
	glPushMatrix();
		// set the OpenGL matrix with the current object matrix
		glLoadMatrixf(&TransformationMatrix[0][0]);
		// apply the rotation in the OpenGL matrix
		glRotated(angle, ax, ay, az);
		// set the object matrix with the new OpenGL matrix
		glGetFloatv(GL_MODELVIEW_MATRIX, &TransformationMatrix[0][0]);
	glPopMatrix();
}
// ************************************************
//	Method: RotateBy(float ang, SmVR_CPoint p)
//	Description: see above function
//
// ************************************************
void SmVR_CGeometricObject::RotateBy(float ang, SmVR_CPoint p)
{
	RotateBy(ang, p.X, p.Y, p.Z);
}
// ************************************************
//	Method: RotateBy(float angle, float ax, float ay, float az, SmVR_CGeometricObject *OCS)
//	Description: perform a rotation using OCS
//	coordinate system as reference for rotating
//
// ************************************************
void SmVR_CGeometricObject::RotateByOnOBJCS(float angle, float ax, float ay, float az, SmVR_CGeometricObject *OCS)
{
	SmVR_CGeometricObject *temp, *parent;

	TestNullSceneGraph(this);

	// create a temporary object
	temp = new SmVR_CGeometricObject("temp");
	temp->parent = NULL;
	// insert it on the object tree
	//SMVR_ROOT_GRAPH->AddChild(temp);

	// copy the GTM of 'OCS' to LTM of 'temp'
	// It makes temp to be exactly in the same position/orientation of OCS
	OCS->CalculateGlobalTransformationMatrix();
	SmVR_CopyMatrix(temp->TransformationMatrix, OCS->GlobalTransformationMatrix);

	// save 'this' parent
	parent = this->parent;

	// Add 'this' object to 'temp' object
	temp->AddChild (this);

	// rotate temp, so it rotates 'this'
	temp->RotateBy(angle,ax,ay, az);

	// recover the previous parent of 'this'
	parent->AddChild(this);

	// remove 'temp'
	temp->Remove(NULL);
	delete temp;
}
// ************************************************
//	Method: RotateByOnOBJCS(float angle, SmVR_CPoint p, SmVR_CGeometricObject *OCS)
//	Description: performa rotation using OCS coordinate system
//	as reference for the rotation
//
// ************************************************
void SmVR_CGeometricObject::RotateByOnOBJCS(float angle, SmVR_CPoint p, SmVR_CGeometricObject *OCS)
{
	RotateByOnOBJCS(angle, p.X, p.Y, p.Z, OCS);
}

/*
// ************************************************
//	Method: GetPosition(void)
//	Description:
//
// ************************************************
SmVR_CPoint SmVR_CGeometricObject::GetPosition(void)
{
	TestNullSceneGraph(this);

	SmVR_CPoint p;

	p.X = TransformationMatrix[3][0];
	p.Y = TransformationMatrix[3][1];
	p.Z = TransformationMatrix[3][2];

	return p;
}

// ************************************************
//	Method:
//	Description:
//
//
//
// ************************************************
void SmVR_CGeometricObject::GetPosition(float *tx, float *ty, float *tz)
{
	SmVR_CPoint p;

	p = GetPosition();

	*tx = p.X;
	*ty = p.Y;
	*tz = p.Z;
}
*/

// ************************************************
//	Method: FindObjectAux(char *name)
//	Description: search for an object called 'name'
//		It looks on child and on brother lists.
//
// ************************************************
SmVR_CGeometricObject *SmVR_CGeometricObject::FindObjectAux(char *name)
{
	SmVR_CGeometricObject *temp = NULL;

   	TestNullSceneGraph(this);

	if (!strcmp(name, this->name))
		return this;

	if (child) {
		temp = child->FindObjectAux(name);
	}
	if (!temp)
		if (brother)
			temp = brother->FindObjectAux(name);

	return temp;
}

// ************************************************
//	Method: FindChildObject(char *name)
//	Description: search for an object just in
//	child 'list'
//
// ************************************************
SmVR_CGeometricObject *SmVR_CGeometricObject::FindChildObject(char *name)
{
   	TestNullSceneGraph(this);

	if (child)
		return child->FindObjectAux(name);
	return NULL;
}
// ************************************************
//	Method: GetParent(void)
//	Description: return a pointer to the first
//	object in the 'child' list
//
// ************************************************
SmVR_CGeometricObject *SmVR_CGeometricObject::GetParent(void)
{
   	TestNullSceneGraph(this);

	return parent;
}
// ************************************************
//	Method: GetFirstChild(void)
//	Description: return a pointer to the first
//	object in the 'child' list
//
// ************************************************
SmVR_CGeometricObject *SmVR_CGeometricObject::GetFirstChild(void)
{
   	TestNullSceneGraph(this);

	return child;
}
// ************************************************
//	Method: GetBrother(void)
//	Description: return a pointer to the first
//	object in the 'brother' list
//
// ************************************************
SmVR_CGeometricObject *SmVR_CGeometricObject::GetBrother(void)
{
   	TestNullSceneGraph(this);

	return brother;
}
// ************************************************
//	Method: GetPreviousBrother(void)
//	Description: return a pointer to the first
//	object in the 'previous brother' list
//
// ************************************************
SmVR_CGeometricObject *SmVR_CGeometricObject::GetPreviousBrother(void)
{
   	TestNullSceneGraph(this);

	return prevBrother;
}
// ************************************************
//	Method: CalculateGlobalTransformationMatrix
//	Description: Calculates the Global transformation
//	matrix of "this" object.
//		To calculate this matrix it multiply the
//	local matrix of this object by its parent and by
//	the parent of its parent and so on, up to the
//	root of the tree
//
// ************************************************

void SmVR_MultMatrix21(SmVR_Matrix m, SmVR_Matrix n, SmVR_Matrix mr);

void SmVR_CGeometricObject::CalculateGlobalTransformationMatrix()
{
	SmVR_CGeometricObject *aux;
	SmVR_Matrix Result;

	// Initialize the GTM with object TM
	SmVR_CopyMatrix (GlobalTransformationMatrix, TransformationMatrix);

	aux = parent; // go from the object's parent ...
	while (aux)   // ... up to the root of the scene graph
	{
		// multiply the current GTM by the parent TM
		SmVR_MultMatrix (GlobalTransformationMatrix, aux->TransformationMatrix, Result); // Res = AC * TM
		//SmVR_MultMatrix21
		// Save the result
		SmVR_CopyMatrix (GlobalTransformationMatrix, Result);
		// go to my parent
		aux = aux->parent;
	}

}
// ************************************************
//	Method: UpdateMatrix()
//	Description: Update the Global Transformation
//	matrix of all objects on the scene graph
// ************************************************
void SmVR_CGeometricObject::UpdateAllMatrices()
{

	glPushMatrix();
		// Apply the local transformation matrix
		glMultMatrixf(&TransformationMatrix[0][0]);
		// Save the global transformation matrix for this object
		glGetFloatv(GL_MODELVIEW_MATRIX, &GlobalTransformationMatrix[0][0]);
		if (child)
			child->UpdateAllMatrices();
	glPopMatrix();

	if (brother)
		brother->UpdateAllMatrices();
}

// ************************************************
//	Method: UpdateAllGlobalTransformationMatrix
//	Description: Start the updating process of all
//	Global Transformation matrices
//
// ************************************************
void SmVR_CGeometricObject::UpdateAllGlobalTransformationMatrix()
{

//   	TestNullSceneGraph(this);
	glPushMatrix();
		// This is used to garantee that the observer trasnformations
		// do not affect the matrices values
		glLoadIdentity();
		SMVR_ROOT_GRAPH->UpdateAllMatrices();
	glPopMatrix();

}

// ************************************************
//	Method: GetLocalTransformationMatrix(SmVR_Matrix m)
//	Description: informs the Local Transformation
//	matrix
//
// ************************************************
void SmVR_CGeometricObject::GetLocalTransformationMatrix(SmVR_Matrix m)
{
    SmVR_CopyMatrix(m,TransformationMatrix);
}
// ************************************************
//	Method: GetGlobalTransformationMatrix(SmVR_Matrix m)
//	Description: informs the GLOBAL Transformation
//	matrix
//
// ************************************************
void SmVR_CGeometricObject::GetGlobalTransformationMatrix(SmVR_Matrix m)
{
	CalculateGlobalTransformationMatrix();
    SmVR_CopyMatrix(m,GlobalTransformationMatrix);
}

// ************************************************
//	Method: SetLocalTransformationMatrixToIdentity(void)
//	Description: set the LTM to identity.
//
// ************************************************
void SmVR_CGeometricObject::SetLocalTransformationMatrixToIdentity(void)
{
	SmVR_SetIdentityMatrix(TransformationMatrix);
}
// ************************************************
//	Method: SetLocalTransformationMatrixToIdentity(SmVR_Matrix m)
//	Description: set the LTM to identity.
//
// ************************************************
void SmVR_CGeometricObject::SetLocalTransformationMatrix(SmVR_Matrix m)
{
	SmVR_CopyMatrix(TransformationMatrix, m);
    /*for (int i=0; i<4;i++)
    {
        for (int j=0; j<4;j++)
            printf("%10.4lf",m[i][j]);
        printf("\n");
    }*/
}

// ************************************************
// Method: SetRotationMatrixToIdent(void)
// Description: set the rotation part of the LTM
// to identity.
//
// ************************************************
void SmVR_CGeometricObject::SetRotationMatrixToIdent(void)
{
	for (int i=0; i<3;i++)
	{
		for (int j=0; j<3;j++)
		{
			if (i==j)
				TransformationMatrix[i][j] = 1;
			else TransformationMatrix[i][j] = 0;
		}
	}
}

// ************************************************
// Method: GetObjectFromFile
// Description: returns ObjectFromFile
// for this object.
//
// ************************************************
SmVR_CObjFromFile* SmVR_CGeometricObject::GetObjectFromFile(void)
{
		return ObjFromFile;
}

// ************************************************
// Method: Collided(SmVR_CObjFromFile *obj)
// Description: returns the number of colliding points
// for this object.
//
// ************************************************
int SmVR_CGeometricObject::Collided(SmVR_CGeometricObject *o, int mode)
{
	  SmVR_Matrix mtx1, mtx2;
	  GetGlobalTransformationMatrix(mtx1);
	  o->GetGlobalTransformationMatrix(mtx2);

	  return ObjFromFile->Collided(o->GetObjectFromFile(),mtx1,mtx2,mode);
}

// ************************************************
// Method: Collided(SmVR_CObjFromFile *obj, int mode)
// Description: returns the number of colliding points
// for this object.
//
// ************************************************
int SmVR_CGeometricObject::Collided(SmVR_CGeometricObject *o)
{
     return  Collided(o,0);
}

// ************************************************
// Method: GetPointInOCS(SmVR_CPoint *in, SmVR_CPoint *out, SmVR_CGeometricObject *OCS)
// Description: converts a point in 'this' CS
// to OCS coordinate system
//
// ************************************************
void SmVR_CGeometricObject::GetPointInOCS(SmVR_CPoint in, SmVR_CPoint *out, SmVR_CGeometricObject *OCS)
{
	SmVR_Matrix inverse;
	GLfloat pWCS[4], pOCS[4];
	int i;

	CalculateGlobalTransformationMatrix();

	// multiply the 'in' point by the GTM and gets it in
	// global coordinate system
	for(i=0;i<4;i++)
	{
	  pWCS[i] = GlobalTransformationMatrix[0][i] * in.X+
			    GlobalTransformationMatrix[1][i] * in.Y+
				GlobalTransformationMatrix[2][i] * in.Z+
				GlobalTransformationMatrix[3][i];
	}

	// Gets the inverse of OCS GTM
   	OCS->CalculateGlobalTransformationMatrix();
	SmVR_InvertMatrix3 (OCS->GlobalTransformationMatrix, inverse);
	//SmVR_InvertMatrix (&OCS->GlobalTransformationMatrix[0][0], &inverse[0][0]);

	// multiply pWCS by the inverse and get it in OCS
	for(i=0;i<4;i++)
	{
	  pOCS[i] = inverse[0][i] * pWCS[0]+
			    inverse[1][i] * pWCS[1]+
			    inverse[2][i] * pWCS[2]+
			    inverse[3][i];
	}

	out->X = pOCS[0];
	out->Y = pOCS[1];
	out->Z = pOCS[2];


}

// ************************************************
//	Function name: *SmVR_Init(char *SceneFileName)
//	Description: function to initialize the SmallVR
//	toolkit.
// ************************************************
SmVR_CGeometricObject *SmVR_Init(char *SceneFileName)
{

	printf("***********************************************\n");
	printf("*               SmallVR Toolkit               *\n");
	printf("*                 Version 3.0                 *\n");
	printf("*  http://grv.inf.pucrs.br                    *\n");
	printf("*                                             *\n");
	printf("*  by Marcio Sarroglia Pinho                  *\n");
	printf("*  pinho@pucrs.br                             *\n");
	printf("*  FACIN - PUCRS - BRAZIL                     *\n");
	printf("***********************************************\n");
	// Create the root object of the scene graph
	SMVR_ROOT_GRAPH = new SmVR_CGeometricObject("SMVR_ROOT_GRAPH");

	if (!SMVR_ROOT_GRAPH)
	{
		printf("Unable to initialize SmallVR");
		exit(1);
	}

//	if (SceneFileName)
//		SMVR_SCENE = new SmVR_Scene(SceneFileName, SMVR_ROOT_GRAPH);

	return SMVR_ROOT_GRAPH;
}
